@@ -171,6 +171,7 @@ struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk);
 struct htx_blk *htx_add_blk(struct htx *htx, enum htx_blk_type type, uint32_t blksz);
 struct htx_blk *htx_remove_blk(struct htx *htx, struct htx_blk *blk);
 void htx_truncate(struct htx *htx, uint32_t offset);
+struct htx_ret htx_drain(struct htx *htx, uint32_t max);
 
 struct htx_blk *htx_replace_blk_value(struct htx *htx, struct htx_blk *blk,
 				      const struct ist old, const struct ist new);
@@ -186,6 +187,7 @@ struct htx_blk *htx_replace_header(struct htx *htx, struct htx_blk *blk,
 				   const struct ist name, const struct ist value);
 
 struct htx_blk *htx_add_header(struct htx *htx, const struct ist name, const struct ist value);
+struct htx_blk *htx_add_blk_type_size(struct htx *htx, enum htx_blk_type type, uint32_t blksz);
 struct htx_blk *htx_add_all_headers(struct htx *htx, const struct http_hdr *hdrs);
 struct htx_blk *htx_add_pseudo_header(struct htx *htx,  enum htx_phdr_type phdr, const struct ist value);
 struct htx_blk *htx_add_endof(struct htx *htx, enum htx_blk_type type);
@@ -689,7 +691,10 @@ static inline size_t buf_room_for_htx_data(const struct buffer *buf)
 
 
 /* Returns an HTX message using the buffer <buf>. Unlike htx_from_buf(), this
- * function does not update to the buffer. */
+ * function does not update to the buffer.
+ * Note that it always returns a valid pointer, either to an initialized buffer
+ * or to the empty buffer.
+ */
 static inline struct htx *htxbuf(const struct buffer *buf)
 {
 	struct htx *htx;
@@ -706,10 +711,10 @@ static inline struct htx *htxbuf(const struct buffer *buf)
 
 /* Returns an HTX message using the buffer <buf>. <buf> is updated to appear as
  * full. It is the caller responsibility to call htx_to_buf() when it finish to
- * manipulate the HTX message to update <buf> accordingly.
+ * manipulate the HTX message to update <buf> accordingly. The returned pointer
+ * is always valid.
  *
- * If the caller can call htxbuf() function to avoir any update of the
- * buffer.
+ * The caller can call htxbuf() function to avoid any update of the buffer.
  */
 static inline struct htx *htx_from_buf(struct buffer *buf)
 {
@@ -730,16 +735,20 @@ static inline void htx_to_buf(struct htx *htx, struct buffer *buf)
 		b_set_data(buf, b_size(buf));
 }
 
-/* Returns 1 if the message is empty, otherwise it returns 0. */
+/* Returns 1 if the message is empty, otherwise it returns 0. Note that it is
+ * illegal to call this with htx == NULL.
+ */
 static inline int htx_is_empty(const struct htx *htx)
 {
-	return (!htx || !htx->used);
+	return !htx->used;
 }
 
-/* Returns 1 if the message is not empty, otherwise it returns 0. */
+/* Returns 1 if the message is not empty, otherwise it returns 0. Note that it
+ * is illegal to call this with htx == NULL.
+ */
 static inline int htx_is_not_empty(const struct htx *htx)
 {
-	return (htx && htx->used);
+	return htx->used;
 }
 
 /* For debugging purpose */
