@@ -676,7 +676,8 @@ static inline void conn_free(struct connection *conn)
 	/* Remove ourself from the session's connections list, if any. */
 	if (!LIST_ISEMPTY(&conn->session_list)) {
 		struct session *sess = conn->owner;
-		sess->resp_conns--;
+		if (conn->flags & CO_FL_SESS_IDLE)
+			sess->idle_conns--;
 		session_unown_conn(sess, conn);
 	}
 
@@ -692,7 +693,8 @@ static inline void conn_free(struct connection *conn)
 	 */
 	if (conn->idle_time > 0) {
 		struct server *srv = __objt_server(conn->target);
-		srv->curr_idle_conns--;
+		HA_ATOMIC_SUB(&srv->curr_idle_conns, 1);
+		srv->curr_idle_thr[tid]--;
 	}
 
 	conn_force_unsubscribe(conn);
@@ -831,9 +833,16 @@ static inline struct wait_event *wl_set_waitcb(struct wait_event *wl, struct tas
 static inline int conn_install_mux(struct connection *conn, const struct mux_ops *mux,
                                    void *ctx, struct proxy *prx, struct session *sess)
 {
+	int ret;
+
 	conn->mux = mux;
 	conn->ctx = ctx;
-	return mux->init ? mux->init(conn, prx, sess) : 0;
+	ret = mux->init ? mux->init(conn, prx, sess) : 0;
+	if (ret < 0) {
+		conn->mux = NULL;
+		conn->ctx = NULL;
+	}
+	return ret;
 }
 
 /* returns a human-readable error code for conn->err_code, or NULL if the code
