@@ -31,12 +31,16 @@
 #endif
 #endif
 
-int shctx_init(struct shared_context **orig_shctx, int maxblocks, int blocksize, int extra, int shared);
-struct shared_block *shctx_row_reserve_hot(struct shared_context *shctx, int data_len);
+int shctx_init(struct shared_context **orig_shctx,
+               int maxblocks, int blocksize, unsigned int maxobjsz,
+               int extra, int shared);
+struct shared_block *shctx_row_reserve_hot(struct shared_context *shctx,
+                                           struct shared_block *last, int data_len);
 void shctx_row_inc_hot(struct shared_context *shctx, struct shared_block *first);
 void shctx_row_dec_hot(struct shared_context *shctx, struct shared_block *first);
 int shctx_row_data_append(struct shared_context *shctx,
-                          struct shared_block *first, unsigned char *data, int len);
+                          struct shared_block *first, struct shared_block *from,
+                          unsigned char *data, int len);
 int shctx_row_data_get(struct shared_context *shctx, struct shared_block *first,
                        unsigned char *dst, int offset, int len);
 
@@ -180,6 +184,19 @@ static inline void _shctx_unlock(struct shared_context *shctx)
 
 /* List Macros */
 
+/*
+ * Insert <s> block after <head> which is not necessarily the head of a list,
+ * so between <head> and the next element after <head>.
+ */
+static inline void shctx_block_append_hot(struct shared_context *shctx,
+                                          struct list *head,
+                                          struct shared_block *s)
+{
+	shctx->nbav--;
+	LIST_DEL(&s->list);
+	LIST_ADD(head, &s->list);
+}
+
 static inline void shctx_block_set_hot(struct shared_context *shctx,
 				    struct shared_block *s)
 {
