@@ -152,8 +152,11 @@ static inline void task_wakeup(struct task *t, unsigned int f)
 	struct eb_root *root = &task_per_thread[tid].rqueue;
 #endif
 
-	state = HA_ATOMIC_OR(&t->state, f);
-	if (!(state & TASK_RUNNING))
+	f |= TASK_QUEUED;
+	state = t->state;
+	while (!HA_ATOMIC_CAS(&t->state, &state, state | f))
+		;
+	if (!(state & TASK_QUEUED))
 		__task_wakeup(t, root);
 }
 
@@ -181,11 +184,14 @@ static inline struct task *__task_unlink_wq(struct task *t)
  */
 static inline struct task *task_unlink_wq(struct task *t)
 {
+	unsigned long locked;
+
 	if (likely(task_in_wq(t))) {
-		if (atleast2(t->thread_mask))
+		locked = atleast2(t->thread_mask);
+		if (locked)
 			HA_SPIN_LOCK(TASK_WQ_LOCK, &wq_lock);
 		__task_unlink_wq(t);
-		if (atleast2(t->thread_mask))
+		if (locked)
 			HA_SPIN_UNLOCK(TASK_WQ_LOCK, &wq_lock);
 	}
 	return t;
@@ -219,14 +225,16 @@ static inline struct task *__task_unlink_rq(struct task *t)
  */
 static inline struct task *task_unlink_rq(struct task *t)
 {
-	if (t->thread_mask != tid_bit)
+	int is_global = t->state & TASK_GLOBAL;
+
+	if (is_global)
 		HA_SPIN_LOCK(TASK_RQ_LOCK, &rq_lock);
 	if (likely(task_in_rq(t))) {
 		if (&t->rq == rq_next)
 			rq_next = eb32sc_next(rq_next, tid_bit);
 		__task_unlink_rq(t);
 	}
-	if (t->thread_mask != tid_bit)
+	if (is_global)
 		HA_SPIN_UNLOCK(TASK_RQ_LOCK, &rq_lock);
 	return t;
 }
@@ -263,7 +271,10 @@ static inline void task_insert_into_tasklet_list(struct task *t)
 	LIST_ADDQ(&task_per_thread[tid].task_list, &tl->list);
 }
 
-static inline void task_remove_from_task_list(struct task *t)
+/* remove the task from the tasklet list. The task MUST already be there. If
+ * unsure, use task_remove_from_task_list() instead.
+ */
+static inline void __task_remove_from_tasklet_list(struct task *t)
 {
 	LIST_DEL(&((struct tasklet *)t)->list);
 	LIST_INIT(&((struct tasklet *)t)->list);
@@ -275,6 +286,12 @@ static inline void task_remove_from_task_list(struct task *t)
 	}
 }
 
+static inline void task_remove_from_tasklet_list(struct task *t)
+{
+	if (likely(!LIST_ISEMPTY(&((struct tasklet *)t)->list)))
+		__task_remove_from_tasklet_list(t);
+}
+
 /*
  * Unlinks the task and adjusts run queue stats.
  * A pointer to the task itself is returned.
