@@ -88,7 +88,7 @@
 #define DNS_FLAG_TRUNCATED      0x0200  /* mask for truncated flag */
 #define DNS_FLAG_REPLYCODE      0x000F  /* mask for reply code */
 
-/* max number of network preference entries are avalaible from the
+/* max number of network preference entries are available from the
  * configuration file.
  */
 #define SRV_MAX_PREF_NET 5
@@ -244,7 +244,7 @@ struct dns_options {
 			struct in6_addr in6;
 		} mask;
 	} pref_net[SRV_MAX_PREF_NET];
-	int pref_net_nb; /* The number of registered prefered networks. */
+	int pref_net_nb; /* The number of registered preferred networks. */
 	int accept_duplicate_ip; /* flag to indicate whether the associated object can use an IP address
 				    already set to an other object of the same group */
 };
@@ -265,7 +265,7 @@ struct dns_resolution {
 	unsigned int          last_valid;          /* time of the last valid response */
 	int                   query_id;            /* DNS query ID dedicated for this resolution */
 	struct eb32_node      qid;                 /* ebtree query id */
-	int                   prefered_query_type; /* prefered query type */
+	int                   prefered_query_type; /* preferred query type */
 	int                   query_type;          /* current query type  */
 	int                   status;              /* status of the resolution being processed RSLV_STATUS_* */
 	int                   step;                /* RSLV_STEP_* */
@@ -292,7 +292,7 @@ struct dns_requester {
 
 /* Last resolution status code */
 enum {
-	RSLV_STATUS_NONE = 0,  /* no resolution occured yet */
+	RSLV_STATUS_NONE = 0,  /* no resolution occurred yet */
 	RSLV_STATUS_VALID,     /* no error */
 	RSLV_STATUS_INVALID,   /* invalid responses */
 	RSLV_STATUS_ERROR,     /* error */
