@@ -41,6 +41,8 @@ extern const char *log_levels[];
 enum {
 	LOG_FORMAT_RFC3164 = 0,
 	LOG_FORMAT_RFC5424,
+	LOG_FORMAT_SHORT,
+	LOG_FORMAT_RAW,
 	LOG_FORMATS,          /* number of supported log formats, must always be last */
 };
 
@@ -50,7 +52,6 @@ enum {
 	LOG_FMT_TEXT = 0, /* raw text */
 	LOG_FMT_EXPR,     /* sample expression */
 	LOG_FMT_SEPARATOR, /* separator replaced by one space */
-	LOG_FMT_VARIABLE,
 
 	/* information fields */
 	LOG_FMT_GLOBAL,
@@ -76,24 +77,29 @@ enum {
 	LOG_FMT_SERVER,
 	LOG_FMT_BYTES,
 	LOG_FMT_BYTES_UP,
-	LOG_FMT_T,
+	LOG_FMT_Ta,
+	LOG_FMT_Th,
+	LOG_FMT_Ti,
 	LOG_FMT_TQ,
 	LOG_FMT_TW,
 	LOG_FMT_TC,
+	LOG_FMT_Tr,
+	LOG_FMT_tr,
+	LOG_FMT_trg,
+	LOG_FMT_trl,
 	LOG_FMT_TR,
+	LOG_FMT_TD,
 	LOG_FMT_TT,
 	LOG_FMT_STATUS,
 	LOG_FMT_CCLIENT,
 	LOG_FMT_CSERVER,
 	LOG_FMT_TERMSTATE,
 	LOG_FMT_TERMSTATE_CK,
-	LOG_FMT_CONN,
 	LOG_FMT_ACTCONN,
 	LOG_FMT_FECONN,
 	LOG_FMT_BECONN,
 	LOG_FMT_SRVCONN,
 	LOG_FMT_RETRIES,
-	LOG_FMT_QUEUES,
 	LOG_FMT_SRVQUEUE,
 	LOG_FMT_BCKQUEUE,
 	LOG_FMT_HDRREQUEST,
@@ -141,6 +147,7 @@ struct logformat_node {
 #define LOG_OPT_REQ_CAP         0x00000008
 #define LOG_OPT_RES_CAP         0x00000010
 #define LOG_OPT_HTTP            0x00000020
+#define LOG_OPT_ESC             0x00000040
 
 
 /* Fields that need to be extracted from the incoming connection or request for
@@ -170,6 +177,7 @@ struct logsrv {
 	int level;
 	int minlvl;
 	int maxlen;
+	struct logsrv *ref;
 };
 
 #endif /* _TYPES_LOG_H */
