@@ -405,6 +405,7 @@ void fd_remove(int fd)
 static inline void fdlist_process_cached_events(volatile struct fdlist *fdlist)
 {
 	int fd, old_fd, e;
+	unsigned long locked;
 
 	for (old_fd = fd = fdlist->first; fd != -1; fd = fdtab[fd].cache.next) {
 		if (fd == -2) {
@@ -421,7 +422,8 @@ static inline void fdlist_process_cached_events(volatile struct fdlist *fdlist)
 			continue;
 
 		HA_ATOMIC_OR(&fd_cache_mask, tid_bit);
-		if (atleast2(fdtab[fd].thread_mask) && HA_SPIN_TRYLOCK(FD_LOCK, &fdtab[fd].lock)) {
+		locked = atleast2(fdtab[fd].thread_mask);
+		if (locked && HA_SPIN_TRYLOCK(FD_LOCK, &fdtab[fd].lock)) {
 			activity[tid].fd_lock++;
 			continue;
 		}
@@ -436,13 +438,13 @@ static inline void fdlist_process_cached_events(volatile struct fdlist *fdlist)
 			fdtab[fd].ev |= FD_POLL_OUT;
 
 		if (fdtab[fd].iocb && fdtab[fd].owner && fdtab[fd].ev) {
-			if (atleast2(fdtab[fd].thread_mask))
+			if (locked)
 				HA_SPIN_UNLOCK(FD_LOCK, &fdtab[fd].lock);
 			fdtab[fd].iocb(fd);
 		}
 		else {
 			fd_release_cache_entry(fd);
-			if (atleast2(fdtab[fd].thread_mask))
+			if (locked)
 				HA_SPIN_UNLOCK(FD_LOCK, &fdtab[fd].lock);
 		}
 	}
@@ -528,6 +530,7 @@ int init_pollers()
 
 	if ((polled_mask = calloc(global.maxsock, sizeof(unsigned long))) == NULL)
 		goto fail_polledmask;
+
 	if ((fdinfo = calloc(global.maxsock, sizeof(struct fdinfo))) == NULL)
 		goto fail_info;
 
@@ -557,15 +560,13 @@ int init_pollers()
 			return 1;
 		}
 	} while (!bp || bp->pref == 0);
-	return 0;
 
- fail_cache:
 	free(fdinfo);
  fail_info:
-	free(fdtab);
- fail_tab:
 	free(polled_mask);
  fail_polledmask:
+	free(fdtab);
+ fail_tab:
 	return 0;
 }
 
