@@ -208,18 +208,24 @@ static inline void h1_release_buf(struct h1c *h1c, struct buffer *bptr)
 	}
 }
 
-static int h1_avail_streams(struct connection *conn)
+/* returns the number of streams in use on a connection to figure if it's
+ * idle or not. We can't have an h1s without a CS so checking h1s is fine,
+ * as the caller will want to know if it was the last one after a detach().
+ */
+static int h1_used_streams(struct connection *conn)
 {
 	struct h1c *h1c = conn->ctx;
 
-	return h1c->h1s ? 0 : 1;
+	return h1c->h1s ? 1 : 0;
 }
 
-static int h1_max_streams(struct connection *conn)
+/* returns the number of streams still available on a connection */
+static int h1_avail_streams(struct connection *conn)
 {
-	return 1;
+	return 1 - h1_used_streams(conn);
 }
 
+
 /*****************************************************************/
 /* functions below are dedicated to the mux setup and management */
 /*****************************************************************/
@@ -780,13 +786,15 @@ static void h1_update_res_conn_hdr(struct h1s *h1s, struct h1m *h1m,
 			if (htx)
 				h1_remove_conn_hdrs(h1m, htx);
 		}
-		if (!(h1m->flags & (H1_MF_VER_11|H1_MF_CONN_KAL))) {
+		if (!(h1m->flags & H1_MF_CONN_KAL) &&
+		    !((h1m->flags & h1s->req.flags) & H1_MF_VER_11)) {
 			if (conn_val)
 				*conn_val = ist("keep-alive");
 			if (htx)
 				h1_add_conn_hdr(h1m, htx, ist("keep-alive"));
 		}
-		if ((h1m->flags & (H1_MF_VER_11|H1_MF_CONN_KAL)) == (H1_MF_VER_11|H1_MF_CONN_KAL)) {
+		else if ((h1m->flags & H1_MF_CONN_KAL) &&
+		         ((h1m->flags & h1s->req.flags) & H1_MF_VER_11)) {
 			if (conn_val)
 				*conn_val = ist("");
 			if (htx)
@@ -1027,9 +1035,11 @@ static size_t h1_process_headers(struct h1s *h1s, struct h1m *h1m, struct htx *h
 		sl->info.res.status = h1s->status;
 	}
 
-	if (h1m->state == H1_MSG_DONE)
+	if (h1m->state == H1_MSG_DONE) {
 		if (!htx_add_endof(htx, HTX_BLK_EOM))
 			goto error;
+		h1s->cs->flags |= CS_FL_EOI;
+	}
 
 	h1_process_conn_mode(h1s, h1m, htx, NULL);
 
@@ -1064,12 +1074,17 @@ static size_t h1_process_headers(struct h1s *h1s, struct h1m *h1m, struct htx *h
  */
 static size_t h1_process_data(struct h1s *h1s, struct h1m *h1m, struct htx *htx,
 			      struct buffer *buf, size_t *ofs, size_t max,
-			      struct buffer *htxbuf)
+			      struct buffer *htxbuf, size_t reserve)
 {
-	uint32_t data_space = htx_free_data_space(htx);
+	uint32_t data_space;
 	size_t total = 0;
 	int ret = 0;
 
+	data_space = htx_free_data_space(htx);
+	if (data_space <= reserve)
+		goto end;
+	data_space -= reserve;
+
 	if (h1m->flags & H1_MF_XFER_LEN) {
 		if (h1m->flags & H1_MF_CLEN) {
 			/* content-length: read only h2m->body_len */
@@ -1119,6 +1134,7 @@ static size_t h1_process_data(struct h1s *h1s, struct h1m *h1m, struct htx *htx,
 				if (!htx_add_endof(htx, HTX_BLK_EOM))
 					goto end;
 				h1m->state = H1_MSG_DONE;
+				h1s->cs->flags |= CS_FL_EOI;
 			}
 		}
 		else if (h1m->flags & H1_MF_CHNK) {
@@ -1175,6 +1191,10 @@ static size_t h1_process_data(struct h1s *h1s, struct h1m *h1m, struct htx *htx,
 				}
 				if (!h1m->curr_len) {
 					h1m->state = H1_MSG_CHUNK_CRLF;
+					data_space = htx_free_data_space(htx);
+					if (data_space <= reserve)
+						goto end;
+					data_space -= reserve;
 					goto new_chunk;
 				}
 				goto end;
@@ -1213,6 +1233,7 @@ static size_t h1_process_data(struct h1s *h1s, struct h1m *h1m, struct htx *htx,
 				if (!htx_add_endof(htx, HTX_BLK_EOM))
 					goto end;
 				h1m->state = H1_MSG_DONE;
+				h1s->cs->flags |= CS_FL_EOI;
 			}
 		}
 		else {
@@ -1222,6 +1243,7 @@ static size_t h1_process_data(struct h1s *h1s, struct h1m *h1m, struct htx *htx,
 			if (!htx_add_endof(htx, HTX_BLK_EOM))
 				goto end;
 			h1m->state = H1_MSG_DONE;
+			h1s->cs->flags |= CS_FL_EOI;
 		}
 	}
 	else {
@@ -1301,24 +1323,13 @@ static size_t h1_process_input(struct h1c *h1c, struct buffer *buf, int flags)
 	struct h1s *h1s = h1c->h1s;
 	struct h1m *h1m;
 	struct htx *htx;
+	size_t data = 0;
 	size_t total = 0;
 	size_t ret = 0;
-	size_t count, max;
+	size_t count, rsv;
 	int errflag;
 
 	htx = htx_from_buf(buf);
-	count = b_data(&h1c->ibuf);
-	max = htx_free_space(htx);
-	if (flags & CO_RFL_KEEP_RSV) {
-		if (max < global.tune.maxrewrite)
-			goto end;
-		max -= global.tune.maxrewrite;
-	}
-	if (count > max)
-		count = max;
-
-	if (!count)
-		goto end;
 
 	if (!conn_is_back(h1c->conn)) {
 		h1m = &h1s->req;
@@ -1329,6 +1340,12 @@ static size_t h1_process_input(struct h1c *h1c, struct buffer *buf, int flags)
 		errflag = H1S_F_RES_ERROR;
 	}
 
+	data = htx->data;
+	count = b_data(&h1c->ibuf);
+	if (!count)
+		goto end;
+	rsv = ((flags & CO_RFL_KEEP_RSV) ? global.tune.maxrewrite : 0);
+
 	do {
 		if (h1m->state <= H1_MSG_LAST_LF) {
 			ret = h1_process_headers(h1s, h1m, htx, &h1c->ibuf, &total, count);
@@ -1336,7 +1353,7 @@ static size_t h1_process_input(struct h1c *h1c, struct buffer *buf, int flags)
 				break;
 		}
 		else if (h1m->state <= H1_MSG_TRAILERS) {
-			ret = h1_process_data(h1s, h1m, htx, &h1c->ibuf, &total, count, buf);
+			ret = h1_process_data(h1s, h1m, htx, &h1c->ibuf, &total, count, buf, rsv);
 			htx = htx_from_buf(buf);
 			if (!ret)
 				break;
@@ -1346,7 +1363,7 @@ static size_t h1_process_input(struct h1c *h1c, struct buffer *buf, int flags)
 			break;
 		}
 		else if (h1m->state == H1_MSG_TUNNEL) {
-			ret = h1_process_data(h1s, h1m, htx, &h1c->ibuf, &total, count, buf);
+			ret = h1_process_data(h1s, h1m, htx, &h1c->ibuf, &total, count, buf, rsv);
 			htx = htx_from_buf(buf);
 			if (!ret)
 				break;
@@ -1366,7 +1383,7 @@ static size_t h1_process_input(struct h1c *h1c, struct buffer *buf, int flags)
 
   end:
 	htx_to_buf(htx, buf);
-
+	data = (htx->data - data);
 	if (h1c->flags & H1C_F_IN_FULL && buf_room_for_htx_data(&h1c->ibuf)) {
 		h1c->flags &= ~H1C_F_IN_FULL;
 		tasklet_wakeup(h1c->wait_event.task);
@@ -1383,9 +1400,11 @@ static size_t h1_process_input(struct h1c *h1c, struct buffer *buf, int flags)
 
 	if ((h1s->cs->flags & CS_FL_REOS) && (!b_data(&h1c->ibuf) || htx_is_empty(htx))) {
 		h1s->cs->flags |= CS_FL_EOS;
+		if (h1m->state > H1_MSG_LAST_LF && h1m->state < H1_MSG_DONE)
+			h1s->cs->flags |= CS_FL_ERROR;
 	}
 
-	return total;
+	return data;
 
   parsing_err:
 	b_reset(&h1c->ibuf);
@@ -1511,6 +1530,8 @@ static size_t h1_process_output(struct h1c *h1c, struct buffer *buf, size_t coun
 				if (!htx_reqline_to_h1(sl, tmp))
 					goto copy;
 				h1m->flags |= H1_MF_XFER_LEN;
+				if (sl->flags & HTX_SL_F_BODYLESS)
+					h1m->flags |= H1_MF_CLEN;
 				h1m->state = H1_MSG_HDR_FIRST;
 				break;
 
@@ -1538,7 +1559,8 @@ static size_t h1_process_output(struct h1c *h1c, struct buffer *buf, size_t coun
 				if (isteqi(n, ist("transfer-encoding")))
 					h1_parse_xfer_enc_header(h1m, v);
 				else if (isteqi(n, ist("content-length"))) {
-					if (h1_parse_cont_len_header(h1m, &v) <= 0)
+					/* Only skip C-L header with invalid value. */
+					if (h1_parse_cont_len_header(h1m, &v) < 0)
 						goto skip_hdr;
 				}
 				else if (isteqi(n, ist("connection"))) {
@@ -1575,8 +1597,12 @@ static size_t h1_process_output(struct h1c *h1c, struct buffer *buf, size_t coun
 					}
 				}
 
-				if ((h1m->flags & (H1_MF_VER_11|H1_MF_RESP|H1_MF_CLEN|H1_MF_CHNK|H1_MF_XFER_LEN)) ==
-				    (H1_MF_VER_11|H1_MF_RESP|H1_MF_XFER_LEN)) {
+				if (((h1m->flags & (H1_MF_VER_11|H1_MF_RESP|H1_MF_CLEN|H1_MF_CHNK|H1_MF_XFER_LEN)) ==
+				     (H1_MF_VER_11|H1_MF_XFER_LEN)) ||
+				    (h1s->status >= 200 && h1s->status != 204 && h1s->status != 304 &&
+				     h1s->meth != HTTP_METH_HEAD && !(h1s->meth == HTTP_METH_CONNECT && h1s->status == 200) &&
+				     (h1m->flags & (H1_MF_VER_11|H1_MF_RESP|H1_MF_CLEN|H1_MF_CHNK|H1_MF_XFER_LEN)) ==
+				     (H1_MF_VER_11|H1_MF_RESP|H1_MF_XFER_LEN))) {
 					/* chunking needed but header not seen */
 					if (!chunk_memcat(tmp, "transfer-encoding: chunked\r\n", 28))
 						goto copy;
@@ -1657,7 +1683,7 @@ static size_t h1_process_output(struct h1c *h1c, struct buffer *buf, size_t coun
 	/* when the output buffer is empty, tmp shares the same area so that we
 	 * only have to update pointers and lengths.
 	 */
-	if (tmp->area == h1c->obuf.area)
+	if (tmp->area == h1c->obuf.area + h1c->obuf.head)
 		h1c->obuf.data = tmp->data;
 	else
 		b_putblk(&h1c->obuf, tmp->area, tmp->data);
@@ -1844,7 +1870,7 @@ static int h1_process(struct h1c * h1c)
 
 	if (!h1s) {
 		if (h1c->flags & H1C_F_CS_ERROR   ||
-		    conn->flags & CO_FL_ERROR     ||
+		    conn->flags & (CO_FL_ERROR | CO_FL_SOCK_WR_SH) ||
 		    conn_xprt_read0_pending(conn))
 			goto release;
 		if (!conn_is_back(conn) && !(h1c->flags & (H1C_F_CS_SHUTW_NOW|H1C_F_CS_SHUTDOWN))) {
@@ -1861,13 +1887,13 @@ static int h1_process(struct h1c * h1c)
 
 	if (!b_data(&h1c->ibuf) && h1s && h1s->cs && h1s->cs->data_cb->wake &&
 	    (conn_xprt_read0_pending(conn) || h1c->flags & H1C_F_CS_ERROR ||
-	    conn->flags & CO_FL_ERROR)) {
+	    conn->flags & (CO_FL_ERROR | CO_FL_SOCK_WR_SH))) {
 		int flags = 0;
 
 		if (h1c->flags & H1C_F_CS_ERROR || conn->flags & CO_FL_ERROR)
 			flags |= CS_FL_ERROR;
 		if (conn_xprt_read0_pending(conn))
-			flags |= CS_FL_EOS;
+			flags |= CS_FL_REOS;
 		h1s->cs->flags |= flags;
 		h1s->cs->data_cb->wake(h1s->cs);
 	}
@@ -2346,7 +2372,7 @@ const struct mux_ops mux_h1_ops = {
 	.detach      = h1_detach,
 	.destroy     = h1_destroy,
 	.avail_streams = h1_avail_streams,
-	.max_streams = h1_max_streams,
+	.used_streams = h1_used_streams,
 	.rcv_buf     = h1_rcv_buf,
 	.snd_buf     = h1_snd_buf,
 #if defined(CONFIG_HAP_LINUX_SPLICE)
