@@ -39,8 +39,6 @@
 #include <proto/log.h>
 #include <proto/hdr_idx.h>
 #include <proto/mux_pt.h>
-#include <proto/proto_tcp.h>
-#include <proto/proto_http.h>
 #include <proto/proxy.h>
 #include <proto/session.h>
 #include <proto/stream.h>
@@ -184,9 +182,6 @@ int intencode(uint64_t i, char **str) {
 	int idx = 0;
 	unsigned char *msg;
 
-	if (!*str)
-		return 0;
-
 	msg = (unsigned char *)*str;
 	if (i < 240) {
 		msg[0] = (unsigned char)i;
@@ -265,7 +260,7 @@ static inline void peer_set_update_msg_type(char *msg_type, int use_identifier,
 /*
  * This prepare the data update message on the stick session <ts>, <st> is the considered
  * stick table.
- *  <msg> is a buffer of <size> to recieve data message content
+ *  <msg> is a buffer of <size> to receive data message content
  * If function returns 0, the caller should consider we were unable to encode this message (TODO:
  * check size)
  */
@@ -377,7 +372,7 @@ static int peer_prepare_updatemsg(struct stksess *ts, struct shared_table *st, u
 
 /*
  * This prepare the switch table message to targeted share table <st>.
- *  <msg> is a buffer of <size> to recieve data message content
+ *  <msg> is a buffer of <size> to receive data message content
  * If function returns 0, the caller should consider we were unable to encode this message (TODO:
  * check size)
  */
@@ -385,7 +380,7 @@ static int peer_prepare_switchmsg(struct shared_table *st, char *msg, size_t siz
 {
 	int len;
 	unsigned short datalen;
-	struct chunk *chunk;
+	struct buffer *chunk;
 	char *cursor, *datamsg, *chunkp, *chunkq;
 	uint64_t data = 0;
 	unsigned int data_type;
@@ -411,7 +406,7 @@ static int peer_prepare_switchmsg(struct shared_table *st, char *msg, size_t siz
 	intencode(st->table->key_size, &cursor);
 
 	chunk = get_trash_chunk();
-	chunkp = chunkq = chunk->str;
+	chunkp = chunkq = chunk->area;
 	/* encode available known data types in table */
 	for (data_type = 0 ; data_type < STKTABLE_DATA_TYPES ; data_type++) {
 		if (st->table->data_ofs[data_type]) {
@@ -435,9 +430,9 @@ static int peer_prepare_switchmsg(struct shared_table *st, char *msg, size_t siz
 	intencode(st->table->expire, &cursor);
 
 	if (chunkq > chunkp) {
-		chunk->len = chunkq - chunkp;
-		memcpy(cursor, chunk->str, chunk->len);
-		cursor += chunk->len;
+		chunk->data = chunkq - chunkp;
+		memcpy(cursor, chunk->area, chunk->data);
+		cursor += chunk->data;
 	}
 
 	/* Compute datalen */
@@ -459,7 +454,7 @@ static int peer_prepare_switchmsg(struct shared_table *st, char *msg, size_t siz
 /*
  * This prepare the acknowledge message on the stick session <ts>, <st> is the considered
  * stick table.
- *  <msg> is a buffer of <size> to recieve data message content
+ *  <msg> is a buffer of <size> to receive data message content
  * If function returns 0, the caller should consider we were unable to encode this message (TODO:
  * check size)
  */
@@ -508,6 +503,9 @@ static void peer_session_release(struct appctx *appctx)
 
 	/* peer session identified */
 	if (peer) {
+		if (appctx->st0 == PEER_SESS_ST_WAITMSG)
+			HA_ATOMIC_SUB(&connected_peers, 1);
+		HA_ATOMIC_SUB(&active_peers, 1);
 		HA_SPIN_LOCK(PEER_LOCK, &peer->lock);
 		if (peer->appctx == appctx) {
 			/* Re-init current table pointers to force announcement on re-connect */
@@ -572,45 +570,50 @@ static void peer_io_handler(struct appctx *appctx)
 	int repl = 0;
 	size_t proto_len = strlen(PEER_SESSION_PROTO_NAME);
 	unsigned int maj_ver, min_ver;
+	int prev_state;
 
-	/* Check if the input buffer is avalaible. */
-	if (si_ic(si)->buf->size == 0)
+	/* Check if the input buffer is available. */
+	if (si_ic(si)->buf.size == 0)
 		goto full;
 
 	while (1) {
+		prev_state = appctx->st0;
 switchstate:
 		maj_ver = min_ver = (unsigned int)-1;
 		switch(appctx->st0) {
 			case PEER_SESS_ST_ACCEPT:
+				prev_state = appctx->st0;
 				appctx->ctx.peers.ptr = NULL;
 				appctx->st0 = PEER_SESS_ST_GETVERSION;
 				/* fall through */
 			case PEER_SESS_ST_GETVERSION:
-				reql = co_getline(si_oc(si), trash.str, trash.size);
+				prev_state = appctx->st0;
+				reql = co_getline(si_oc(si), trash.area,
+						  trash.size);
 				if (reql <= 0) { /* closed or EOL not found */
 					if (reql == 0)
 						goto out;
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				if (trash.str[reql-1] != '\n') {
+				if (trash.area[reql-1] != '\n') {
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				else if (reql > 1 && (trash.str[reql-2] == '\r'))
-					trash.str[reql-2] = 0;
+				else if (reql > 1 && (trash.area[reql-2] == '\r'))
+					trash.area[reql-2] = 0;
 				else
-					trash.str[reql-1] = 0;
+					trash.area[reql-1] = 0;
 
 				co_skip(si_oc(si), reql);
 
 				/* test protocol */
-				if (strncmp(PEER_SESSION_PROTO_NAME " ", trash.str, proto_len + 1) != 0) {
+				if (strncmp(PEER_SESSION_PROTO_NAME " ", trash.area, proto_len + 1) != 0) {
 					appctx->st0 = PEER_SESS_ST_EXIT;
 					appctx->st1 = PEER_SESS_SC_ERRPROTO;
 					goto switchstate;
 				}
-				if (peer_get_version(trash.str + proto_len + 1, &maj_ver, &min_ver) == -1 ||
+				if (peer_get_version(trash.area + proto_len + 1, &maj_ver, &min_ver) == -1 ||
 				    maj_ver != PEER_MAJOR_VER || min_ver > PEER_MINOR_VER) {
 					appctx->st0 = PEER_SESS_ST_EXIT;
 					appctx->st1 = PEER_SESS_SC_ERRVERSION;
@@ -620,26 +623,28 @@ switchstate:
 				appctx->st0 = PEER_SESS_ST_GETHOST;
 				/* fall through */
 			case PEER_SESS_ST_GETHOST:
-				reql = co_getline(si_oc(si), trash.str, trash.size);
+				prev_state = appctx->st0;
+				reql = co_getline(si_oc(si), trash.area,
+						  trash.size);
 				if (reql <= 0) { /* closed or EOL not found */
 					if (reql == 0)
 						goto out;
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				if (trash.str[reql-1] != '\n') {
+				if (trash.area[reql-1] != '\n') {
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				else if (reql > 1 && (trash.str[reql-2] == '\r'))
-					trash.str[reql-2] = 0;
+				else if (reql > 1 && (trash.area[reql-2] == '\r'))
+					trash.area[reql-2] = 0;
 				else
-					trash.str[reql-1] = 0;
+					trash.area[reql-1] = 0;
 
 				co_skip(si_oc(si), reql);
 
 				/* test hostname match */
-				if (strcmp(localpeer, trash.str) != 0) {
+				if (strcmp(localpeer, trash.area) != 0) {
 					appctx->st0 = PEER_SESS_ST_EXIT;
 					appctx->st1 = PEER_SESS_SC_ERRHOST;
 					goto switchstate;
@@ -649,27 +654,30 @@ switchstate:
 				/* fall through */
 			case PEER_SESS_ST_GETPEER: {
 				char *p;
-				reql = co_getline(si_oc(si), trash.str, trash.size);
+
+				prev_state = appctx->st0;
+				reql = co_getline(si_oc(si), trash.area,
+						  trash.size);
 				if (reql <= 0) { /* closed or EOL not found */
 					if (reql == 0)
 						goto out;
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				if (trash.str[reql-1] != '\n') {
+				if (trash.area[reql-1] != '\n') {
 					/* Incomplete line, we quit */
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				else if (reql > 1 && (trash.str[reql-2] == '\r'))
-					trash.str[reql-2] = 0;
+				else if (reql > 1 && (trash.area[reql-2] == '\r'))
+					trash.area[reql-2] = 0;
 				else
-					trash.str[reql-1] = 0;
+					trash.area[reql-1] = 0;
 
 				co_skip(si_oc(si), reql);
 
 				/* parse line "<peer name> <pid> <relative_pid>" */
-				p = strchr(trash.str, ' ');
+				p = strchr(trash.area, ' ');
 				if (!p) {
 					appctx->st0 = PEER_SESS_ST_EXIT;
 					appctx->st1 = PEER_SESS_SC_ERRPROTO;
@@ -679,7 +687,7 @@ switchstate:
 
 				/* lookup known peer */
 				for (curpeer = curpeers->remote; curpeer; curpeer = curpeer->next) {
-					if (strcmp(curpeer->id, trash.str) == 0)
+					if (strcmp(curpeer->id, trash.area) == 0)
 						break;
 				}
 
@@ -717,11 +725,13 @@ switchstate:
 				curpeer->appctx = appctx;
 				appctx->ctx.peers.ptr = curpeer;
 				appctx->st0 = PEER_SESS_ST_SENDSUCCESS;
+				HA_ATOMIC_ADD(&active_peers, 1);
 				/* fall through */
 			}
 			case PEER_SESS_ST_SENDSUCCESS: {
 				struct shared_table *st;
 
+				prev_state = appctx->st0;
 				if (!curpeer) {
 					curpeer = appctx->ctx.peers.ptr;
 					HA_SPIN_LOCK(PEER_LOCK, &curpeer->lock);
@@ -730,8 +740,10 @@ switchstate:
 						goto switchstate;
 					}
 				}
-				repl = snprintf(trash.str, trash.size, "%d\n", PEER_SESS_SC_SUCCESSCODE);
-				repl = ci_putblk(si_ic(si), trash.str, repl);
+				repl = snprintf(trash.area, trash.size,
+						"%d\n",
+						PEER_SESS_SC_SUCCESSCODE);
+				repl = ci_putblk(si_ic(si), trash.area, repl);
 				if (repl <= 0) {
 					if (repl == -1)
 						goto full;
@@ -778,11 +790,12 @@ switchstate:
 
 
 				/* switch to waiting message state */
+				HA_ATOMIC_ADD(&connected_peers, 1);
 				appctx->st0 = PEER_SESS_ST_WAITMSG;
 				goto switchstate;
 			}
 			case PEER_SESS_ST_CONNECT: {
-
+				prev_state = appctx->st0;
 				if (!curpeer) {
 					curpeer = appctx->ctx.peers.ptr;
 					HA_SPIN_LOCK(PEER_LOCK, &curpeer->lock);
@@ -793,7 +806,7 @@ switchstate:
 				}
 
 				/* Send headers */
-				repl = snprintf(trash.str, trash.size,
+				repl = snprintf(trash.area, trash.size,
 				                PEER_SESSION_PROTO_NAME " %u.%u\n%s\n%s %d %d\n",
 				                PEER_MAJOR_VER,
 				                (curpeer->flags & PEER_F_DWNGRD) ? PEER_DWNGRD_MINOR_VER : PEER_MINOR_VER,
@@ -807,7 +820,7 @@ switchstate:
 					goto switchstate;
 				}
 
-				repl = ci_putblk(si_ic(si), trash.str, repl);
+				repl = ci_putblk(si_ic(si), trash.area, repl);
 				if (repl <= 0) {
 					if (repl == -1)
 						goto full;
@@ -822,6 +835,7 @@ switchstate:
 			case PEER_SESS_ST_GETSTATUS: {
 				struct shared_table *st;
 
+				prev_state = appctx->st0;
 				if (!curpeer) {
 					curpeer = appctx->ctx.peers.ptr;
 					HA_SPIN_LOCK(PEER_LOCK, &curpeer->lock);
@@ -834,27 +848,28 @@ switchstate:
 				if (si_ic(si)->flags & CF_WRITE_PARTIAL)
 					curpeer->statuscode = PEER_SESS_SC_CONNECTEDCODE;
 
-				reql = co_getline(si_oc(si), trash.str, trash.size);
+				reql = co_getline(si_oc(si), trash.area,
+						  trash.size);
 				if (reql <= 0) { /* closed or EOL not found */
 					if (reql == 0)
 						goto out;
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				if (trash.str[reql-1] != '\n') {
+				if (trash.area[reql-1] != '\n') {
 					/* Incomplete line, we quit */
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
-				else if (reql > 1 && (trash.str[reql-2] == '\r'))
-					trash.str[reql-2] = 0;
+				else if (reql > 1 && (trash.area[reql-2] == '\r'))
+					trash.area[reql-2] = 0;
 				else
-					trash.str[reql-1] = 0;
+					trash.area[reql-1] = 0;
 
 				co_skip(si_oc(si), reql);
 
 				/* Register status code */
-				curpeer->statuscode = atoi(trash.str);
+				curpeer->statuscode = atoi(trash.area);
 
 				/* Awake main task */
 				task_wakeup(curpeers->sync_task, TASK_WOKEN_MSG);
@@ -898,17 +913,19 @@ switchstate:
 					appctx->st0 = PEER_SESS_ST_END;
 					goto switchstate;
 				}
+				HA_ATOMIC_ADD(&connected_peers, 1);
 				appctx->st0 = PEER_SESS_ST_WAITMSG;
 				/* fall through */
 			}
 			case PEER_SESS_ST_WAITMSG: {
 				struct stksess *ts, *newts = NULL;
 				uint32_t msg_len = 0;
-				char *msg_cur = trash.str;
-				char *msg_end = trash.str;
+				char *msg_cur = trash.area;
+				char *msg_end = trash.area;
 				unsigned char msg_head[7];
 				int totl = 0;
 
+				prev_state = appctx->st0;
 				if (!curpeer) {
 					curpeer = appctx->ctx.peers.ptr;
 					HA_SPIN_LOCK(PEER_LOCK, &curpeer->lock);
@@ -976,7 +993,10 @@ switchstate:
 							goto switchstate;
 						}
 
-						reql = co_getblk(si_oc(si), trash.str, msg_len, totl);
+						reql = co_getblk(si_oc(si),
+								 trash.area,
+								 msg_len,
+								 totl);
 						if (reql <= 0) /* closed */
 							goto incomplete;
 						totl += reql;
@@ -1441,7 +1461,9 @@ incomplete:
 						if (st->last_get != st->last_acked) {
 							int msglen;
 
-							msglen = peer_prepare_ackmsg(st, trash.str, trash.size);
+							msglen = peer_prepare_ackmsg(st,
+										     trash.area,
+										     trash.size);
 							if (!msglen) {
 								/* internal error: message does not fit in trash */
 								appctx->st0 = PEER_SESS_ST_END;
@@ -1449,7 +1471,9 @@ incomplete:
 							}
 
 							/* message to buffer */
-							repl = ci_putblk(si_ic(si), trash.str, msglen);
+							repl = ci_putblk(si_ic(si),
+									 trash.area,
+									 msglen);
 							if (repl <= 0) {
 								/* no more write possible */
 								if (repl == -1) {
@@ -1471,7 +1495,9 @@ incomplete:
 								if (st != curpeer->last_local_table) {
 									int msglen;
 
-									msglen = peer_prepare_switchmsg(st, trash.str, trash.size);
+									msglen = peer_prepare_switchmsg(st,
+													trash.area,
+													trash.size);
 									if (!msglen) {
 										HA_SPIN_UNLOCK(STK_TABLE_LOCK, &st->table->lock);
 										/* internal error: message does not fit in trash */
@@ -1480,7 +1506,9 @@ incomplete:
 									}
 
 									/* message to buffer */
-									repl = ci_putblk(si_ic(si), trash.str, msglen);
+									repl = ci_putblk(si_ic(si),
+											 trash.area,
+											 msglen);
 									if (repl <= 0) {
 										HA_SPIN_UNLOCK(STK_TABLE_LOCK, &st->table->lock);
 										/* no more write possible */
@@ -1520,7 +1548,11 @@ incomplete:
 									ts->ref_cnt++;
 									HA_SPIN_UNLOCK(STK_TABLE_LOCK, &st->table->lock);
 
-									msglen = peer_prepare_updatemsg(ts, st, updateid, trash.str, trash.size, new_pushed, 0);
+									msglen = peer_prepare_updatemsg(ts, st, updateid,
+													trash.area,
+													trash.size,
+													new_pushed,
+													0);
 									if (!msglen) {
 										/* internal error: message does not fit in trash */
 										HA_SPIN_LOCK(STK_TABLE_LOCK, &st->table->lock);
@@ -1531,7 +1563,9 @@ incomplete:
 									}
 
 									/* message to buffer */
-									repl = ci_putblk(si_ic(si), trash.str, msglen);
+									repl = ci_putblk(si_ic(si),
+											 trash.area,
+											 msglen);
 									if (repl <= 0) {
 										/* no more write possible */
 										HA_SPIN_LOCK(STK_TABLE_LOCK, &st->table->lock);
@@ -1563,7 +1597,9 @@ incomplete:
 								if (st != curpeer->last_local_table) {
 									int msglen;
 
-									msglen = peer_prepare_switchmsg(st, trash.str, trash.size);
+									msglen = peer_prepare_switchmsg(st,
+													trash.area,
+													trash.size);
 									if (!msglen) {
 										/* internal error: message does not fit in trash */
 										appctx->st0 = PEER_SESS_ST_END;
@@ -1571,7 +1607,9 @@ incomplete:
 									}
 
 									/* message to buffer */
-									repl = ci_putblk(si_ic(si), trash.str, msglen);
+									repl = ci_putblk(si_ic(si),
+											 trash.area,
+											 msglen);
 									if (repl <= 0) {
 										/* no more write possible */
 										if (repl == -1) {
@@ -1608,7 +1646,11 @@ incomplete:
 									HA_SPIN_UNLOCK(STK_TABLE_LOCK, &st->table->lock);
 
 									use_timed = !(curpeer->flags & PEER_F_DWNGRD);
-									msglen = peer_prepare_updatemsg(ts, st, updateid, trash.str, trash.size, new_pushed, use_timed);
+									msglen = peer_prepare_updatemsg(ts, st, updateid,
+													trash.area,
+													trash.size,
+													new_pushed,
+													use_timed);
 									if (!msglen) {
 										/* internal error: message does not fit in trash */
 										HA_SPIN_LOCK(STK_TABLE_LOCK, &st->table->lock);
@@ -1619,7 +1661,9 @@ incomplete:
 									}
 
 									/* message to buffer */
-									repl = ci_putblk(si_ic(si), trash.str, msglen);
+									repl = ci_putblk(si_ic(si),
+											 trash.area,
+											 msglen);
 									if (repl <= 0) {
 										/* no more write possible */
 										HA_SPIN_LOCK(STK_TABLE_LOCK, &st->table->lock);
@@ -1647,7 +1691,9 @@ incomplete:
 								if (st != curpeer->last_local_table) {
 									int msglen;
 
-									msglen = peer_prepare_switchmsg(st, trash.str, trash.size);
+									msglen = peer_prepare_switchmsg(st,
+													trash.area,
+													trash.size);
 									if (!msglen) {
 										/* internal error: message does not fit in trash */
 										appctx->st0 = PEER_SESS_ST_END;
@@ -1655,7 +1701,9 @@ incomplete:
 									}
 
 									/* message to buffer */
-									repl = ci_putblk(si_ic(si), trash.str, msglen);
+									repl = ci_putblk(si_ic(si),
+											 trash.area,
+											 msglen);
 									if (repl <= 0) {
 										/* no more write possible */
 										if (repl == -1) {
@@ -1691,7 +1739,11 @@ incomplete:
 									HA_SPIN_UNLOCK(STK_TABLE_LOCK, &st->table->lock);
 
 									use_timed = !(curpeer->flags & PEER_F_DWNGRD);
-									msglen = peer_prepare_updatemsg(ts, st, updateid, trash.str, trash.size, new_pushed, use_timed);
+									msglen = peer_prepare_updatemsg(ts, st, updateid,
+													trash.area,
+													trash.size,
+													new_pushed,
+													use_timed);
 									if (!msglen) {
 										/* internal error: message does not fit in trash */
 										HA_SPIN_LOCK(STK_TABLE_LOCK, &st->table->lock);
@@ -1702,7 +1754,9 @@ incomplete:
 									}
 
 									/* message to buffer */
-									repl = ci_putblk(si_ic(si), trash.str, msglen);
+									repl = ci_putblk(si_ic(si),
+											 trash.area,
+											 msglen);
 									if (repl <= 0) {
 										/* no more write possible */
 										HA_SPIN_LOCK(STK_TABLE_LOCK, &st->table->lock);
@@ -1775,14 +1829,21 @@ incomplete:
 				goto out;
 			}
 			case PEER_SESS_ST_EXIT:
-				repl = snprintf(trash.str, trash.size, "%d\n", appctx->st1);
-				if (ci_putblk(si_ic(si), trash.str, repl) == -1)
+				if (prev_state == PEER_SESS_ST_WAITMSG)
+					HA_ATOMIC_SUB(&connected_peers, 1);
+				prev_state = appctx->st0;
+				repl = snprintf(trash.area, trash.size,
+						"%d\n", appctx->st1);
+				if (ci_putblk(si_ic(si), trash.area, repl) == -1)
 					goto full;
 				appctx->st0 = PEER_SESS_ST_END;
 				goto switchstate;
 			case PEER_SESS_ST_ERRSIZE: {
 				unsigned char msg[2];
 
+				if (prev_state == PEER_SESS_ST_WAITMSG)
+					HA_ATOMIC_SUB(&connected_peers, 1);
+				prev_state = appctx->st0;
 				msg[0] = PEER_MSG_CLASS_ERROR;
 				msg[1] = PEER_MSG_ERR_SIZELIMIT;
 
@@ -1794,15 +1855,22 @@ incomplete:
 			case PEER_SESS_ST_ERRPROTO: {
 				unsigned char msg[2];
 
+				if (prev_state == PEER_SESS_ST_WAITMSG)
+					HA_ATOMIC_SUB(&connected_peers, 1);
+				prev_state = appctx->st0;
 				msg[0] = PEER_MSG_CLASS_ERROR;
 				msg[1] = PEER_MSG_ERR_PROTOCOL;
 
 				if (ci_putblk(si_ic(si), (char *)msg, sizeof(msg)) == -1)
 					goto full;
 				appctx->st0 = PEER_SESS_ST_END;
+				prev_state = appctx->st0;
 				/* fall through */
 			}
 			case PEER_SESS_ST_END: {
+				if (prev_state == PEER_SESS_ST_WAITMSG)
+					HA_ATOMIC_SUB(&connected_peers, 1);
+				prev_state = appctx->st0;
 				if (curpeer) {
 					HA_SPIN_UNLOCK(PEER_LOCK, &curpeer->lock);
 					curpeer = NULL;
@@ -1821,7 +1889,7 @@ out:
 		HA_SPIN_UNLOCK(PEER_LOCK, &curpeer->lock);
 	return;
 full:
-	si_applet_cant_put(si);
+	si_rx_room_blk(si);
 	goto out;
 }
 
@@ -1849,6 +1917,8 @@ static void peer_session_forceshutdown(struct appctx *appctx)
 	if (appctx->applet != &peer_applet)
 		return;
 
+	if (appctx->st0 == PEER_SESS_ST_WAITMSG)
+		HA_ATOMIC_SUB(&connected_peers, 1);
 	appctx->st0 = PEER_SESS_ST_END;
 	appctx_wakeup(appctx);
 }
@@ -1907,7 +1977,7 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
 	s->flags = SF_ASSIGNED|SF_ADDR_SET;
 
 	/* applet is waiting for data */
-	si_applet_cant_get(&s->si[0]);
+	si_cant_get(&s->si[0]);
 	appctx_wakeup(appctx);
 
 	/* initiate an outgoing connection */
@@ -1923,12 +1993,14 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
 	if (unlikely((cs = cs_new(conn)) == NULL))
 		goto out_free_conn;
 
-	conn_prepare(conn, peer->proto, peer->xprt);
-	conn_install_mux(conn, &mux_pt_ops, cs);
-	si_attach_cs(&s->si[1], cs);
-
 	conn->target = s->target = &s->be->obj_type;
 	memcpy(&conn->addr.to, &peer->addr, sizeof(conn->addr.to));
+
+	conn_prepare(conn, peer->proto, peer->xprt);
+	if (conn_install_mux(conn, &mux_pt_ops, cs, s->be, NULL) < 0)
+		goto out_free_cs;
+	si_attach_cs(&s->si[1], cs);
+
 	s->do_log = NULL;
 	s->uniq_id = 0;
 
@@ -1936,9 +2008,12 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
 
 	peer->appctx = appctx;
 	task_wakeup(s->task, TASK_WOKEN_INIT);
+	HA_ATOMIC_ADD(&active_peers, 1);
 	return appctx;
 
 	/* Error unrolling */
+out_free_cs:
+	cs_free(cs);
  out_free_conn:
 	conn_free(conn);
  out_free_strm:
@@ -1956,9 +2031,9 @@ static struct appctx *peer_session_create(struct peers *peers, struct peer *peer
  * Task processing function to manage re-connect and peer session
  * tasks wakeup on local update.
  */
-static struct task *process_peer_sync(struct task * task)
+static struct task *process_peer_sync(struct task * task, void *context, unsigned short state)
 {
-	struct peers *peers = task->context;
+	struct peers *peers = context;
 	struct peer *ps;
 	struct shared_table *st;
 
@@ -2075,8 +2150,8 @@ static struct task *process_peer_sync(struct task * task)
 	} /* !stopping */
 	else {
 		/* soft stop case */
-		if (task->state & TASK_WOKEN_SIGNAL) {
-			/* We've just recieved the signal */
+		if (state & TASK_WOKEN_SIGNAL) {
+			/* We've just received the signal */
 			if (!(peers->flags & PEERS_F_DONOTSTOP)) {
 				/* add DO NOT STOP flag if not present */
 				HA_ATOMIC_ADD(&jobs, 1);
