@@ -1264,6 +1264,13 @@ int smp_resolve_args(struct proxy *p)
 				break;
 			}
 
+			if (p->bind_proc & ~px->bind_proc) {
+				ha_alert("parsing [%s:%d] : stick-table '%s' not present on all processes covered by proxy '%s'.\n",
+					 cur->file, cur->line, px->id, p->id);
+				cfgerr++;
+				break;
+			}
+
 			free(arg->data.str.area);
 			arg->data.str.area = NULL;
 			arg->unresolved = 0;
