@@ -62,7 +62,7 @@ struct session *session_new(struct proxy *fe, struct listener *li, enum obj_type
 		HA_ATOMIC_ADD(&totalconn, 1);
 		HA_ATOMIC_ADD(&jobs, 1);
 		LIST_INIT(&sess->srv_list);
-		sess->resp_conns = 0;
+		sess->idle_conns = 0;
 	}
 	return sess;
 }
@@ -87,6 +87,7 @@ void session_free(struct session *sess)
 				LIST_DEL(&conn->session_list);
 				LIST_INIT(&conn->session_list);
 				conn->owner = NULL;
+				conn->flags &= ~CO_FL_SESS_IDLE;
 				if (!srv_add_to_idle_list(objt_server(conn->target), conn))
 					conn->mux->destroy(conn);
 			} else {
